{# TEMPLATE VAR SETTINGS #}
{%- set url_root = pathto('', 1) %}
{%- if url_root == '#' %}{% set url_root = '' %}{% endif %}
{%- if not embedded and docstitle %}
  {%- set titlesuffix = " &mdash; "|safe + docstitle|e %}
{%- else %}
  {%- set titlesuffix = "" %}
{%- endif %}
{%- set lang_attr = 'en' if language == None else (language | replace('_', '-')) %}
{% import 'theme_variables.jinja' as theme_variables %}

<!DOCTYPE html>
<!--[if IE 8]><html class="no-js lt-ie9" lang="{{ lang_attr }}" > <![endif]-->
<!--[if gt IE 8]><!--> <html class="no-js" lang="{{ lang_attr }}" > <!--<![endif]-->
<head>
  <meta charset="utf-8">
  {{ metatags }}
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  {% block htmltitle %}
  <title>{{ title|striptags|e }}{{ titlesuffix }}</title>
  {% endblock %}

  {# FAVICON #}
  {% if favicon %}
    <link rel="shortcut icon" href="{{ pathto('_static/' + favicon, 1) }}"/>
  {% endif %}
  {# CANONICAL URL #}
  {% if theme_canonical_url %}
    <link rel="canonical" href="{{ theme_canonical_url }}{{ pagename }}.html"/>
  {% endif %}

  {# CSS #}

  {# OPENSEARCH #}
  {% if not embedded %}
    {% if use_opensearch %}
      <link rel="search" type="application/opensearchdescription+xml"
            title="{% trans docstitle=docstitle|e %}Search within {{ docstitle }}{% endtrans %}"
            href="{{ pathto('_static/opensearch.xml', 1) }}"/>
    {% endif %}

  {% endif %}

  <link rel="stylesheet" href="{{ pathto('_static/' + style, 1) }}" type="text/css" />
  <!-- <link rel="stylesheet" href="{{ pathto('_static/pygments.css', 1) }}" type="text/css" /> -->
  {%- for css in css_files %}
    {%- if css|attr("rel") %}
  <link rel="{{ css.rel }}" href="{{ pathto(css.filename, 1) }}" type="text/css"{% if css.title is not none %} title="{{ css.title }}"{% endif %} />
    {%- else %}
  <link rel="stylesheet" href="{{ pathto(css, 1) }}" type="text/css" />
    {%- endif %}
  {%- endfor %}
  {%- for cssfile in extra_css_files %}
    <link rel="stylesheet" href="{{ pathto(cssfile, 1) }}" type="text/css" />
  {%- endfor %}

  {%- block linktags %}
    {%- if hasdoc('about') %}
    <link rel="author" title="{{ _('About these documents') }}" href="{{ pathto('about') }}" />
    {%- endif %}
    {%- if hasdoc('genindex') %}
    <link rel="index" title="{{ _('Index') }}" href="{{ pathto('genindex') }}" />
    {%- endif %}
    {%- if hasdoc('search') %}
    <link rel="search" title="{{ _('Search') }}" href="{{ pathto('search') }}" />
    {%- endif %}
    {%- if hasdoc('copyright') %}
    <link rel="copyright" title="{{ _('Copyright') }}" href="{{ pathto('copyright') }}" />
    {%- endif %}
    {%- if next %}
    <link rel="next" title="{{ next.title|striptags|e }}" href="{{ next.link|e }}" />
    {%- endif %}
    {%- if prev %}
    <link rel="prev" title="{{ prev.title|striptags|e }}" href="{{ prev.link|e }}" />
    {%- endif %}
  {%- endblock %}
  {%- block extrahead %} {% endblock %}

  {# Keep google analytics in head#}
  <script async src="https://www.googletagmanager.com/gtag/js?id=UA-167094275-1"></script>
  <script src="{{ pathto('_static/js/googleanalytics.min.js', 1) }}"></script>
  {# Keep modernizr in head - http://modernizr.com/docs/#installing #}
  <script src="{{ pathto('_static/js/modernizr.min.js', 1) }}"></script>

  {% include "fonts.html" %}
</head>

<div class="container-fluid header-holder tutorials-header" id="header-holder">
  <div class="container">
    <div class="header-container">
<!--      <img src="../_static/battery_logo.png" width="7%">-->
<!--      <img src="/Users/jambo/Documents/PhD/code/modules/tsbatteries/docs/_static/battery_logo.png" width="5%">-->
      <img src="https://github.com/jambo6/batteries/blob/main/docs/_static/battery_logo.png?raw=true" width="5%">
      <b>tsbatteries documentation</b>
    </div>
  </div>
</div>


<body class="catalyst-body">

  {% block extrabody %} {% endblock %}

    {# SIDE NAV, TOGGLES ON MOBILE #}

    <div class="table-of-contents-link-wrapper">
      <span>Table of Contents</span>
      <a href="#" class="toggle-table-of-contents" data-behavior="toggle-table-of-contents"></a>
    </div>

    <nav data-toggle="wy-nav-shift" class="catalyst-left-menu" id="catalyst-left-menu">
      <div class="catalyst-side-scroll">
        <div class="catalyst-menu catalyst-menu-vertical" data-spy="affix" role="navigation" aria-label="main navigation">
          <div class="catalyst-left-menu-search">
            {% block sidebartitle %}

            {% if theme_display_version %}
              {%- set nav_version = version %}
              {% if READTHEDOCS and current_version %}
                {%- set nav_version = current_version %}
              {% endif %}
              {% if nav_version %}
                <div class="version">
                  <a href="https://catalyst-team.github.io/catalyst/versions.html">{{ nav_version }}</a>
                </div>
              {% endif %}
            {% endif %}

            {% include "searchbox.html" %}

            {% endblock %}
          </div>

          {% block menu %}
            {#
              The singlehtml builder doesn't handle this toctree call when the
              toctree is empty. Skip building this for now.
            #}
            {% if 'singlehtml' not in builder %}
              {% set global_toc = toctree(maxdepth=1,
                                          collapse=theme_collapse_navigation|tobool,
                                          includehidden=theme_includehidden|tobool,
                                          titles_only=theme_titles_only|tobool) %}
            {% endif %}
            {% if global_toc %}
              {{ global_toc }}
            {% else %}
              <!-- Local TOC -->
              <div class="local-toc">{{ toc }}</div>
            {% endif %}
          {% endblock %}
        </div>
      </div>
    </nav>

    <div class="catalyst-container">
      <div class="catalyst-page-level-bar" id="catalyst-page-level-bar">
        <div class="catalyst-breadcrumbs-wrapper">
          {% include "breadcrumbs.html" %}
        </div>

        <div class="catalyst-shortcuts-wrapper" id="catalyst-shortcuts-wrapper">
          Shortcuts
        </div>
      </div>

      <section data-toggle="wy-nav-shift" id="catalyst-content-wrap" class="catalyst-content-wrap">
        <div class="catalyst-content-left">

        {%- block content %}
          {% if theme_style_external_links|tobool %}
          <div class="rst-content style-external-links">
          {% else %}
          <div class="rst-content">
          {% endif %}
            <div role="main" class="main-content" itemscope="itemscope" itemtype="http://schema.org/Article">
            {%- block document %}
             <article itemprop="articleBody" id="catalyst-article" class="catalyst-article">
              {% block body %}{% endblock %}
             </article>
             {% if self.comments()|trim %}
             <div class="articleComments">
              {% block comments %}{% endblock %}
             </div>
             {% endif%}
            </div>
            {%- endblock %}
            {% include "footer.html" %}
          </div>
        {%- endblock %}
        </div>

        <div class="catalyst-content-right" id="catalyst-content-right">
          <div class="catalyst-right-menu" id="catalyst-right-menu">
            <div class="catalyst-side-scroll" id="catalyst-side-scroll-right">
              {{ toc }}
            </div>
          </div>
        </div>
      </section>
    </div>

  {% include "versions.html" %}

  {% if not embedded %}

     {% if sphinx_version >= "1.8.0" %}
       <script type="text/javascript" id="documentation_options" data-url_root="{{ pathto('', 1) }}" src="{{ pathto('_static/documentation_options.js', 1) }}"></script>
       {%- for scriptfile in script_files %}
         {{ js_tag(scriptfile) }}
       {%- endfor %}
     {% else %}
       <script type="text/javascript">
           var DOCUMENTATION_OPTIONS = {
               URL_ROOT:'{{ url_root }}',
               VERSION:'{{ release|e }}',
               LANGUAGE:'{{ language }}',
               COLLAPSE_INDEX:false,
               FILE_SUFFIX:'{{ '' if no_search_suffix else file_suffix }}',
               HAS_SOURCE:  {{ has_source|lower }},
               SOURCELINK_SUFFIX: '{{ sourcelink_suffix }}'
           };
       </script>
       {%- for scriptfile in script_files %}
         <script type="text/javascript" src="{{ pathto(scriptfile, 1) }}"></script>
       {%- endfor %}
     {% endif %}

  {% endif %}

  <script type="text/javascript" src="{{ pathto('_static/js/vendor/popper.min.js', 1) }}"></script>
  <script type="text/javascript" src="{{ pathto('_static/js/vendor/bootstrap.min.js', 1) }}"></script>
  <script type="text/javascript" src="{{ pathto('_static/js/theme.js', 1) }}"></script>

  <script type="text/javascript">
      jQuery(function () {
          SphinxRtdTheme.Navigation.enable({{ 'true' if theme_sticky_navigation|tobool else 'false' }});
      });
  </script>

  {%- block footer %} {% endblock %}

  <!-- Begin Footer -->
  <footer class="site-footer" id="site-footer">
  </footer>


</body>
</html>
